"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compose = void 0;
/**
 * @public
 */
function compose(...middlewares) {
    if (!Array.isArray(middlewares))
        throw new TypeError("Middleware stack must be an array!");
    for (const fn of middlewares) {
        if (typeof fn !== "function")
            throw new TypeError("Middleware must be composed of functions!");
    }
    if (middlewares.length == 1) {
        return middlewares[0];
    }
    return function (context, next) {
        // last called middleware #
        let index = -1;
        return dispatch(0);
        async function dispatch(i) {
            if (i <= index) {
                throw new Error("next() called multiple times");
            }
            index = i;
            let fn = middlewares[i];
            if (i === middlewares.length)
                fn = next;
            if (!fn)
                return {};
            return fn(context, dispatch.bind(null, i + 1));
        }
    };
}
exports.compose = compose;
//# sourceMappingURL=middleware.js.map