"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServerHandler = void 0;
const logic_1 = require("./logic");
const middleware_1 = require("./middleware");
// @internal
function createServerHandler() {
    let middlewares;
    let theFinalHandler;
    function doMiddlewareComposition() {
        theFinalHandler = (0, middleware_1.compose)(...middlewares);
    }
    function resetMiddlewares() {
        middlewares = (0, logic_1.getDefaultMiddlewares)();
        doMiddlewareComposition();
    }
    // initialize default middleware
    resetMiddlewares();
    const use = async (handler) => {
        middlewares.push(handler);
        doMiddlewareComposition();
    };
    async function processRequest(currentContext, req) {
        const ctx = (0, logic_1.contextFromRequest)(currentContext, req);
        const res = await theFinalHandler(ctx, logic_1.defaultHandler);
        return (0, logic_1.normalizeResponseBody)(req, res);
    }
    return {
        resetMiddlewares,
        use,
        processRequest,
    };
}
exports.createServerHandler = createServerHandler;
//# sourceMappingURL=server-handler.js.map