import { IBaseComponent, IConfigComponent, IHttpServerComponent } from "@well-known-components/interfaces";
/**
 * @beta
 */
export type StandardStatusResponse = {
    status: "pass" | "fail" | "warn";
    version?: string;
    releaseId?: string;
    notes?: string[];
    output?: string;
    serviceId?: string;
    description?: string;
    details: Record<string, StandardStatusResponseDetail>;
};
/**
 * @beta
 */
export type StandardStatusResponseDetail = {
    status: "pass" | "fail" | "warn";
    componentType?: string;
    componentId?: string;
};
/**
 * Binds status checks to the server
 *  - GET /health/ready - readyness probe
 *  - GET /health/startup - startup probe
 *  - GET /health/live - liveness probe
 * @public
 */
export declare function createStatusCheckComponent<Context extends object = {}>(components: {
    server: IHttpServerComponent<Context>;
    config: IConfigComponent;
}): Promise<IBaseComponent>;
